#!/usr/bin/perl
#Majid Masso
#School of Systems Biology, George Mason University
#April 2013
#Filter in CA ATOM lines of protein chain
#Model 1 selected in NMR files, and one atomic alternate location letter used
#Filter out chains with one or more non-standard amino acids
#Can run as stand-alone with or without creation of an output file
#Used by all other programs to create a filtered PDB file prior to their implementation
#Output is directly piped into the other programs without creating an output file
$arg=0;
for($i=0;$i<@ARGV;$i++){
	if($ARGV[$i] eq "-e"){
		$pdbfile=$ARGV[$i+1];
		$arg++;
	}
	elsif($ARGV[$i] eq "-o"){
		$outfile=$ARGV[$i+1];
	}
}
if($arg!=1){die "Usage: perl residue_filter_automute.pl -e <PDB_file> -o <output_file>\n";}
$pdb=lc(substr($pdbfile,0,4)); $chain=uc(substr($pdbfile,4,1));
if($chain eq ''){die "Missing a protein chain letter at the end of the PDB code\n";}
@allres=(ALA,CYS,ASP,GLU,PHE,GLY,HIS,ILE,LYS,LEU,MET,ASN,PRO,GLN,ARG,SER,THR,VAL,TRP,TYR);
$c=0;$d=0;
open(PDB,".//pdb$pdb.ent")  or die "\nCannot locate PDB file $pdb\n";
if($outfile eq ''){
	while(<PDB>){
		chomp($_);
		$at[$c]=substr($_,13,2);$altloc[$c]=substr($_,16,1);$aa[$c]=substr($_,17,3);$chn[$c]=substr($_,21,1);$resnum[$c]=substr($_,22,4);$ctr=0;
		if(($_=~/^ATOM/ || $_=~/^HETATM/) && $chain eq $chn[$c] && (($altloc[$c] eq ' ' || $altloc[$c] eq '1') || ($altloc[$c] eq 'A' || $altloc[$c] eq 'D')) && $at[$c] eq 'CA'){
			unless($c==0){
				$value = $resnum[$c]-$resnum[$c-1];
				if($value > 1){die "\nCannot generate predictions due to gap in the protein structure file $pdb after sequence position number $resnum[$c-1]\n";}
			}
			for($i=0;$i<20;$i++){
				if($aa[$c] eq $allres[$i]){
					$ctr++; last;
				}
			}
			if($ctr==0){die "\nCannot generate predictions for mutations in protein structure file $pdb due to non-standard amino acid residue $aa[$c] at position number $resnum[$c]\n";}
			$line[$c]=$_;
            	$c++;
		}
		if($_=~/^ENDMDL/ || ($_=~/^TER/ && $aa[$c] eq $aa[$c-1] && $chain eq $chn[$c] && $resnum[$c]==$resnum[$c-1])){last;}
		if($_=~/^HELIX/ || $_=~/^SHEET/){$ssline[$d]=$_;$d++;}
	}
	close(PDB);
for($k=0;$k<$d;$k++){print "$ssline[$k]\n";}
for($j=0;$j<$c;$j++){print "$line[$j]\n";}
}
else{
	open(OUT,">$outfile");
	while(<PDB>){
		chomp($_);
		$at[$c]=substr($_,13,2);$altloc[$c]=substr($_,16,1);$aa[$c]=substr($_,17,3);$chn[$c]=substr($_,21,1);$resnum[$c]=substr($_,22,4);$ctr=0;
		if(($_=~/^ATOM/ || $_=~/^HETATM/) && $chain eq $chn[$c] && (($altloc[$c] eq ' ' || $altloc[$c] eq '1') || ($altloc[$c] eq 'A' || $altloc[$c] eq 'D')) && $at[$c] eq 'CA'){
			unless($c==0){
				$value = $resnum[$c]-$resnum[$c-1];
				if($value > 1){die "\nCannot generate predictions due to gap in the protein structure file $pdb after sequence position number $resnum[$c-1]\n";}
			}
			for($i=0;$i<20;$i++){
				if($aa[$c] eq $allres[$i]){
					$ctr++; last;
				}
			}
			if($ctr==0){die "\nCannot generate predictions for mutations in protein structure file $pdb due to non-standard amino acid residue $aa[$c] at position number $resnum[$c]\n";}
			$line[$c]=$_;
            	$c++;
		}
		if($_=~/^ENDMDL/ || ($_=~/^TER/ && $aa[$c] eq $aa[$c-1] && $chain eq $chn[$c] && $resnum[$c]==$resnum[$c-1])){last;}
		if($_=~/^HELIX/ || $_=~/^SHEET/){$ssline[$d]=$_;$d++;}
	}
	close(PDB);
for($k=0;$k<$d;$k++){print OUT "$ssline[$k]\n";}
for($j=0;$j<$c;$j++){print OUT "$line[$j]\n";}
}
close(OUT);