#!/usr/bin/perl
#Majid Masso
#School of Systems Biology, George Mason University
#April 2013
use LWP::Simple;
$arg=0;
for($i=0;$i<@ARGV;$i++){
	if($ARGV[$i] eq "-t"){
		$strtype=$ARGV[$i+1];
		$arg++;
	}
	elsif($ARGV[$i] eq "-f"){
		$mutfile=$ARGV[$i+1];
		$arg++;
	}
	elsif($ARGV[$i] eq "-o"){
		$outfile=$ARGV[$i+1];
	}
}
if($arg!=2){die "\n\nUsage: perl human_nsSNPs.pl -t <type_of_structure_files> -f <mutant_data_filename> -o <output_filename>\n\nThe protein structure coordinate files for which mutant predictions are requested in any single program run must either ALL be from the PDB (type 0), which are automatically obtained (internet access is required) and saved in the current working directory by this program as pdbXXXX.ent (XXXX is a lowercase 4-character PDB ID); or, they must be any combination of manually obtained PDB files and the user's own model structures (type 1) in exact PDB-type coordinate file format with respect to HELIX, SHEET, and ATOM lines, with these actual PDB files and the user's own model files placed by the user in the current working directory using the naming convention pdbXXXX.ent (user should select the XXXX to be lowercase 4-character PDB-like IDs to distinguish their personal model structures). For user-submitted model structure files, details on formatting of HELIX and SHEET lines consistent with the PDB can be found at http://www.wwpdb.org/documentation/format32/sect5.html, and details on formatting ATOM lines are available at http://www.wwpdb.org/documentation/format32/sect9.html#ATOM\n\nThe mutant data file should be a text file in the current working directory, each line containing 2 space- or tab-delimited items: the 4-character PDB ID with one protein chain letter attached (e.g., 1egcA), and the mutation (e.g. I53T). The PDB IDs in this file must either correspond to actual structures deposited in PDB, or else they must be 4-character lowercase PDB-like IDs (XXXX) selected by the user in order to name their personal model structures.\n\nThe underlying model for predicting whether a mutation is either neutral or disease-associated is based on the Random Forest classification algorithm.\n\nThe flag for an output filename is optional; excluding it will send the results to the standard output (i.e., the command prompt screen).\n\n";}
open(PROFS,">test_vector.arff");
print PROFS "\@relation nsSNP_test_profiles\n\n\@attribute wt {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute sub {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute residual real\n\@attribute ec1 real\n\@attribute dist1 real\n\@attribute aa1 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute ec2 real\n\@attribute dist2 real\n\@attribute aa2 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute ec3 real\n\@attribute dist3 real\n\@attribute aa3 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute ec4 real\n\@attribute dist4 real\n\@attribute aa4 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute ec5 real\n\@attribute dist5 real\n\@attribute aa5 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute ec6 real\n\@attribute dist6 real\n\@attribute aa6 {A, C, D, E, F, G, H, I, K, L, M, N, P, Q, R, S, T, V, W, Y}\n\@attribute volume real\n\@attribute sT real\n\@attribute depth {S, U, B}\n\@attribute count real\n\@attribute secstr {H, S, C, T}\n\@attribute type {nt, da}\n\n\@data\n";
%aalist=(A=>ALA,C=>CYS,D=>ASP,E=>GLU,F=>PHE,G=>GLY,H=>HIS,I=>ILE,K=>LYS,L=>LEU,M=>MET,N=>ASN,P=>PRO,Q=>GLN,R=>ARG,S=>SER,T=>THR,V=>VAL,W=>TRP,Y=>TYR);
@single=keys(%aalist);
open(DATA, "$mutfile") or die "\nCannot locate mutant data file $mutfile in current working directory\n";
while(<DATA>){
	chomp($_);
	($code,$mutation)=split; $mutation=uc($mutation);
	$pdb=lc(substr($code,0,4));$pdbalt=uc($pdb);$chain=uc(substr($code,4,1));$code=$pdb.$chain;
	$size=length($mutation);$altsize=$size-2;$altaltsize=$size-1;
	$wt=uc(substr($mutation,0,1));$pos=substr($mutation,1,$altsize);$sub=uc(substr($mutation,$altaltsize,1));
	if($strtype==0){getstore("http://www.rcsb.org/pdb/files/$pdbalt.pdb", ".//pdb$pdb.ent");}
	elsif($strtype==1){
		open(TEST, "pdb$pdb.ent") or die "\nCannot locate file pdb$pdb.ent corresponding to either the PDB ID or the user's own model PDB-like ID $pdb; before continuing, please deposit this file in the current working directory, or else remove all mutants associated with this structure from the submitted mutant data file $mutfile\n"; close(TEST);}
	else{die "\nYou must choose a correct structure file type number with the -t flag, either 0 for all PDB files that will be automatically downloaded by this program (internet access required) into the current working directory, or 1 for any combination of actual PDB structures and user's own models that are all manually placed by the user into the current working directory\n";}
	$error=0;
	for($i=0;$i<@single;$i++){
		if($sub eq $single[$i]){
			$error++;
			last;
		}
	}
	@diffprof=();@newprof=();
	open(LOC,"perl .//AutoMute2_support_programs//potential_profile_automute.pl -p potential_1417_cut12.out -e $code -c 12.0 |");
	while(<LOC>){
		chomp($_);
		@line=split(/\s+/,$_);
		$abort=0;
		if($line[0] eq $aalist{$wt} && $line[1]==$pos && $wt ne $sub && $error!=0){
			$abort++;
			@extract=($line[4],$line[5],$line[6],$line[7],$line[8]);
			last;
		}
	}
	close(LOC);
	open (MUTPOT, "perl .//AutoMute2_support_programs//mutant_potential_automute.pl -p potential_1417_cut12.out -e $code -m $mutation -c 12.0 |");
	while(<MUTPOT>){
		chomp($_);
		if(/^\s+\d/){
			($line1,$line2,$line3,$line4,$line5,$line6)=split;
			push(@diffprof, $line6);
		}
	}
	close(MUTPOT);
	if($abort==0){next;}
	(@ordered)=&NN($code,$pos);
	for($i=0;$i<21;$i+=3){
		if($ordered[$i+1] != 0){
			$newprof[$i]=$diffprof[$ordered[$i]];
			$newprof[$i+1]=$ordered[$i+1];
			$newprof[$i+2]=$ordered[$i+2];
		}
		elsif($ordered[$i+1]==0 && $ordered[$i+2] ne 'Z'){$newprof[$i]=$diffprof[$ordered[$i]];$newprof[$i+1]=0;$newprof[$i+2]=0;}
		elsif($ordered[$i+1]==0 && $ordered[$i+2] eq 'Z'){$newprof[$i]=0;$newprof[$i+1]=0;$newprof[$i+2]='Z';}
	}
	splice(@newprof,1,2);
	if(($newprof[2] eq 0 && $newprof[3] eq Z) || ($newprof[5] eq 0 && $newprof[6] eq Z) || ($newprof[8] eq 0 && $newprof[9] eq Z) || ($newprof[11] eq 0 && $newprof[12] eq Z) || ($newprof[14] eq 0 && $newprof[15] eq Z) || ($newprof[17] eq 0 && $newprof[18] eq Z)){
		print "\nERROR: mutation $mutation of PDB ID $pdb$chain cannot be tested because position $pos has fewer than 6 neighbors based on the structure tessellation\n";
		next;}
	@left=($wt,$sub);
	@right=(@extract,"nt");
	unshift(@newprof,@left);push(@newprof,@right);
	print PROFS join(',',@newprof),"\n";
	push(@pdb,$pdb);push(@chain,$chain);push(@mutation,$mutation);push(@volume,$extract[0]);push(@sT,$extract[1]);push(@depth,$extract[2]);push(@contact,$extract[3]);push(@secstr,$extract[4]);
}
close(DATA);
close(PROFS);
open(RESULT, 'java -mx1024m weka.classifiers.meta.CostSensitiveClassifier -S 5 -t .//models//masso_complete_da_nt_snp_profiles.arff -C .//models//2by2.cost -T test_vector.arff -p 0 -W weka.classifiers.trees.RandomForest -- -I 100 -K 0 -S 5 |');
#open(RESULT, 'java -mx1024m weka.classifiers.trees.RandomForest -l ".//models//RF_100trees_20CV_complete_cost4_allseeds5.model" -T "test_vector.arff" -p 0 |');
open(OUT,">$outfile");
unless($outfile eq ''){
	print OUT "PDB_ID  Chain  Mutation  Prediction  Confid  Vol   sT  Loc  Num  SS\n";
	$ct1=0;
	while(<RESULT>){
		chomp($_);
		@output=split(/\s+/,$_);
		if(@output!=()){
			if($output[1] eq "da"){$new="Disease";}
			elsif($output[1] eq "nt"){$new="Neutral";}
			printf OUT ("%5.4s%6.1s%10.6s%13.9s%8.2f%6.1f%6.2f%3.1s%5.1d%5.1s\n",$pdb[$ct1],$chain[$ct1],$mutation[$ct1],$new,$output[2],$volume[$ct1],$sT[$ct1],$depth[$ct1],$contact[$ct1],$secstr[$ct1]);
			$ct1++;}
	}
}
else{
	print "\nPDB_ID  Chain  Mutation  Prediction  Confid  Vol   sT  Loc  Num  SS\n";
	$ct1=0;
	while(<RESULT>){
		chomp($_);
		@output=split(/\s+/,$_);
		if(@output!=()){
			if($output[1] eq "da"){$new="Disease";}
			elsif($output[1] eq "nt"){$new="Neutral";}
			printf("%5.4s%6.1s%10.6s%13.9s%8.2f%6.1f%6.2f%3.1s%5.1d%5.1s\n",$pdb[$ct1],$chain[$ct1],$mutation[$ct1],$new,$output[2],$volume[$ct1],$sT[$ct1],$depth[$ct1],$contact[$ct1],$secstr[$ct1]);
			$ct1++;}
	}
}
close(RESULT);
sub NN{
	my(@list)=@_;
	$minv=10000;%m=();%c=();%d=();@w=();@n=();@arr=();%val=();%val2=();%val3=();@ords=();
	open (SIMP, "perl .//AutoMute2_support_programs//simplex_automute.pl -p potential_1417_cut12.out -e $code -c 12.0 |");
	while(<SIMP>){
		chomp($_);
		if (/quadruplets$/){
			($len,$residues,$quads,@z)=split;
		}
		if (/^[A-Z]{4}/){
			($q2,$t,$v1,$v2,$v3,$v4,$L1,$L2,$L3,$L4,$L5,$L6,$L7)=split;
			@v=($v1,$v2,$v3,$v4);
			for (my $i=0;$i<4;$i++){
				if ($v[$i] < $minv){
					$minv=$v[$i];
				}
			}
			if ($L1 <= 12.000 && $L2 <= 12.000 && $L3 <= 12.000 && $L4 <= 12.000 && $L5 <= 12.000 && $L6 <= 12.000){
			$aa1=substr($q2,0,1);$aa2=substr($q2,1,1);$aa3=substr($q2,2,1);$aa4=substr($q2,3,1);
			if($v1==$list[1]){
				$m{$v1}=0;$m{$v2}=$L1;$m{$v3}=$L2;$m{$v4}=$L3;
				$c{$v1}=0;$c{$v2}=$v2-$v1;$c{$v3}=$v3-$v1;$c{$v4}=$v4-$v1;
				$d{$v1}=$aa1;$d{$v2}=$aa2;$d{$v3}=$aa3;$d{$v4}=$aa4;
			}
			elsif($v2==$list[1]){
				$m{$v2}=0;$m{$v1}=$L1;$m{$v3}=$L4;$m{$v4}=$L5;
				$c{$v1}=$v1-$v2;$c{$v2}=0;$c{$v3}=$v3-$v2;$c{$v4}=$v4-$v2;
				$d{$v1}=$aa1;$d{$v2}=$aa2;$d{$v3}=$aa3;$d{$v4}=$aa4;
			}
			elsif($v3==$list[1]){
				$m{$v3}=0;$m{$v1}=$L2;$m{$v2}=$L4;$m{$v4}=$L6;
				$c{$v1}=$v1-$v3;$c{$v2}=$v2-$v3;$c{$v3}=0;$c{$v4}=$v4-$v3;
				$d{$v1}=$aa1;$d{$v2}=$aa2;$d{$v3}=$aa3;$d{$v4}=$aa4;
			}
			elsif($v4==$list[1]){
				$m{$v4}=0;$m{$v1}=$L3;$m{$v2}=$L5;$m{$v3}=$L6;
				$c{$v1}=$v1-$v4;$c{$v2}=$v2-$v4;$c{$v3}=$v3-$v4;$c{$v4}=0;
				$d{$v1}=$aa1;$d{$v2}=$aa2;$d{$v3}=$aa3;$d{$v4}=$aa4;
			}
		}
		}
	}
	close(SIMP);
	for($i=0;$i<$len;$i++){
		if($m{$i+$minv}>0){
			$w[$i]=$m{$i+$minv};
			$val{$w[$i]}=$i+$minv;
			$val2{$w[$i]}=$c{$i+$minv};
			$val3{$w[$i]}=$d{$i+$minv};
		}
		elsif($m{$i+$minv}==0 && $i+$minv==$list[1]){
			$val{0}=$i+$minv;
			$val2{0}=$c{$i+$minv};
			$val3{0}=$d{$i+$minv};
		}
		else{$val{$i+1000}=$i+minv; $val2{$i+1000}=0; $val3{$i+1000}='Z';}
	}
	@n=keys(%val);
	@arr=sort {$a <=> $b} @n;
	for($i=0;$i<$len;$i++){
		@more=($val{$arr[$i]}-$minv,$val2{$arr[$i]},$val3{$arr[$i]});
		push(@ords,@more);
	}
	return (@ords);
}
